"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Puppeteer = void 0;
const puppeteer_extra_1 = __importDefault(require("puppeteer-extra"));
const puppeteer_extra_plugin_stealth_1 = __importDefault(require("puppeteer-extra-plugin-stealth"));
const puppeteer_1 = require("puppeteer");
const Puppeteer_node_options_1 = require("./Puppeteer.node.options");
const DEFAULT_USER_AGENT = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/73.0.3683.75 Safari/537.36';
class Puppeteer {
    constructor() {
        this.description = Puppeteer_node_options_1.nodeDescription;
        this.methods = {
            loadOptions: {
                async getDevices() {
                    const deviceNames = Object.keys(puppeteer_1.devices);
                    const returnData = [];
                    for (const name of deviceNames) {
                        const device = puppeteer_1.devices[name];
                        returnData.push({
                            name,
                            value: name,
                            description: `${device.viewport.width} x ${device.viewport.height} @ ${device.viewport.deviceScaleFactor}x`,
                        });
                    }
                    return returnData;
                }
            }
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const options = this.getNodeParameter('options', 0, {});
        const launchArguments = options.launchArguments || {};
        const operation = this.getNodeParameter('operation', 0);
        const headless = options.headless !== false;
        const stealth = options.stealth === true;
        const pageCaching = options.pageCaching !== false;
        const launchArgs = launchArguments.args;
        const args = [];
        if (launchArgs && launchArgs.length > 0) {
            args.push(...launchArgs.map((arg) => arg.arg));
        }
        if (options.proxyServer) {
            args.push(`--proxy-server=${options.proxyServer}`);
        }
        if (stealth) {
            puppeteer_extra_1.default.use((0, puppeteer_extra_plugin_stealth_1.default)());
        }
        const browser = await puppeteer_extra_1.default.launch({ headless, args });
        for (let itemIndex = 0; itemIndex < items.length; itemIndex++) {
            const urlString = this.getNodeParameter('url', itemIndex);
            const { parameter: someHeaders = [] } = (options.headers || {});
            const { parameter: queryParameters = [] } = this.getNodeParameter('queryParameters', itemIndex);
            const requestHeaders = someHeaders.reduce((acc, cur) => {
                acc[cur.name] = cur.value;
                return acc;
            }, {});
            const device = options.device;
            const url = new URL(urlString);
            const page = await browser.newPage();
            await page.setCacheEnabled(pageCaching);
            if (device) {
                const emulatedDevice = puppeteer_1.devices[device];
                if (emulatedDevice) {
                    await page.emulate(emulatedDevice);
                }
            }
            else {
                const userAgent = requestHeaders['User-Agent'] || requestHeaders['user-agent'] || DEFAULT_USER_AGENT;
                await page.setUserAgent(userAgent);
            }
            await page.setExtraHTTPHeaders(requestHeaders);
            for (const queryParameter of queryParameters) {
                url.searchParams.append(queryParameter.name, queryParameter.value);
            }
            console.log(`Processing ${itemIndex + 1} of ${items.length}: [${operation}]${device ? ` [${device}] ` : ' '}${url}`);
            const waitUntil = options.waitUntil;
            const timeout = options.timeout;
            const response = await page.goto(url.toString(), { waitUntil, timeout });
            const headers = await response.headers();
            const statusCode = response.status();
            let returnItem;
            if (statusCode !== 200) {
                if (this.continueOnFail() !== true) {
                    returnItem = {
                        json: {
                            headers,
                            statusCode,
                        }
                    };
                    if (operation === 'getPageContent') {
                        returnItem.json.body = await page.content();
                    }
                }
                else {
                    throw new Error(`Request failed with status code ${statusCode}`);
                }
            }
            else {
                if (operation === 'getPageContent') {
                    const click = this.getNodeParameter('click', itemIndex);
                    const wait = this.getNodeParameter('wait', itemIndex);
                    const selector = this.getNodeParameter('selector', itemIndex);
                    const css = this.getNodeParameter('css', itemIndex);
                    if (css) {
                        await page.addStyleTag({ content: css });
                    }
                    if (click) {
                        await page.click(click);
                    }
                    if (wait) {
                        await page.waitForTimeout(3000);
                    }
                    if (selector) {
                        await page.waitForSelector(selector);
                    }
                    returnItem = {
                        json: {
                            body: selector ? await page.$(selector) : await page.content(),
                            headers,
                            statusCode,
                        }
                    };
                }
                else if (operation === 'getScreenshot') {
                    const dataPropertyName = this.getNodeParameter('dataPropertyName', itemIndex);
                    const fileName = options.fileName;
                    const type = this.getNodeParameter('imageType', itemIndex);
                    const selector = this.getNodeParameter('selector', itemIndex);
                    const css = this.getNodeParameter('css', itemIndex);
                    const click = this.getNodeParameter('click', itemIndex);
                    const wait = this.getNodeParameter('wait', itemIndex);
                    const fullPage = this.getNodeParameter('fullPage', itemIndex);
                    const screenshotOptions = {
                        type,
                        fullPage,
                    };
                    if (type !== 'png') {
                        const quality = this.getNodeParameter('quality', itemIndex);
                        screenshotOptions.quality = quality;
                    }
                    if (fileName) {
                        screenshotOptions.path = fileName;
                    }
                    let screenshot;
                    if (css) {
                        await page.addStyleTag({ content: css });
                    }
                    if (click) {
                        await page.click(click);
                    }
                    if (wait) {
                        await page.waitForTimeout(3000);
                    }
                    if (selector) {
                        await page.waitForSelector(selector);
                        const element = await page.$(selector);
                        screenshot = await element.screenshot();
                    }
                    else {
                        screenshot = await page.screenshot(screenshotOptions);
                    }
                    if (screenshot) {
                        const binaryData = await this.helpers.prepareBinaryData(screenshot, screenshotOptions.path, `image/${type}`);
                        returnItem = {
                            binary: { [dataPropertyName]: binaryData },
                            json: {
                                headers,
                                statusCode,
                            }
                        };
                    }
                }
                else if (operation === 'getPDF') {
                    const dataPropertyName = this.getNodeParameter('dataPropertyName', itemIndex);
                    const pageRanges = this.getNodeParameter('pageRanges', itemIndex);
                    const displayHeaderFooter = this.getNodeParameter('displayHeaderFooter', itemIndex);
                    const omitBackground = this.getNodeParameter('omitBackground', itemIndex);
                    const printBackground = this.getNodeParameter('printBackground', itemIndex);
                    const landscape = this.getNodeParameter('landscape', itemIndex);
                    const preferCSSPageSize = this.getNodeParameter('preferCSSPageSize', itemIndex);
                    const scale = this.getNodeParameter('scale', itemIndex);
                    const margin = this.getNodeParameter('margin', 0, {});
                    const click = this.getNodeParameter('click', itemIndex);
                    const wait = this.getNodeParameter('wait', itemIndex);
                    const css = this.getNodeParameter('css', itemIndex);
                    const selector = this.getNodeParameter('selector', itemIndex);
                    let headerTemplate;
                    let footerTemplate;
                    let height;
                    let width;
                    let format;
                    if (displayHeaderFooter === true) {
                        headerTemplate = this.getNodeParameter('headerTemplate', itemIndex);
                        footerTemplate = this.getNodeParameter('footerTemplate', itemIndex);
                    }
                    if (preferCSSPageSize !== true) {
                        height = this.getNodeParameter('height', itemIndex);
                        width = this.getNodeParameter('width', itemIndex);
                        if (!height || !width) {
                            format = this.getNodeParameter('format', itemIndex);
                        }
                    }
                    const pdfOptions = {
                        format,
                        displayHeaderFooter,
                        omitBackground,
                        printBackground,
                        landscape,
                        headerTemplate,
                        footerTemplate,
                        preferCSSPageSize,
                        scale,
                        height,
                        width,
                        pageRanges,
                        margin,
                    };
                    const fileName = options.fileName;
                    if (fileName) {
                        pdfOptions.path = fileName;
                    }
                    if (css) {
                        await page.addStyleTag({ content: css });
                    }
                    if (click) {
                        await page.click(click);
                    }
                    if (wait) {
                        await page.waitForTimeout(3000);
                    }
                    let pdf;
                    if (selector) {
                        await page.waitForSelector(selector);
                        const element = await page.$(selector);
                        pdf = await element.pdf(pdfOptions);
                    }
                    else {
                        pdf = await page.pdf(pdfOptions);
                    }
                    if (pdf) {
                        const binaryData = await this.helpers.prepareBinaryData(pdf, pdfOptions.path, 'application/pdf');
                        returnItem = {
                            binary: { [dataPropertyName]: binaryData },
                            json: {
                                headers,
                                statusCode,
                            },
                            pairedItem: {
                                item: itemIndex,
                            },
                        };
                    }
                }
            }
            await page.close();
            if (returnItem) {
                returnData.push(returnItem);
            }
        }
        await browser.close();
        return this.prepareOutputData(returnData);
    }
}
exports.Puppeteer = Puppeteer;
//# sourceMappingURL=Puppeteer.node.js.map